use core::pin::Pin;

use embedded_storage_async::nor_flash::{NorFlash, ReadNorFlash};
use nrf_softdevice::{Flash, FlashError};

const MAX_STRING_LENGTH: u32 = 128;
const CONFIG_START: u32 = 0x80000;

pub enum ConfigItem {
    Ssid,
    WifiPw,
    MqtBroker,
}

pub struct Config<'a> {
    flash: Pin<&'a mut Flash>,
}

impl<'a> Config<'a> {
    pub fn new(flash: Pin<&'a mut Flash>) -> Config<'a> {
        Self { flash }
    }

    pub async fn write(
        &mut self,
        config_item: ConfigItem,
        value: &[u8; MAX_STRING_LENGTH as usize],
    ) -> Result<(), FlashError> {
        let from = CONFIG_START + MAX_STRING_LENGTH * config_item as u32;
        let to = from + MAX_STRING_LENGTH;

        self.flash.as_mut().erase(from, to).await?;
        self.flash.as_mut().write(from, value).await?;

        Ok(())
    }

    pub async fn read(
        &mut self,
        config_item: ConfigItem,
    ) -> Result<[u8; MAX_STRING_LENGTH as usize], FlashError> {
        let offset = CONFIG_START + MAX_STRING_LENGTH * config_item as u32;
        let mut buffer: [u8; MAX_STRING_LENGTH as usize] = [0; MAX_STRING_LENGTH as usize];
        self.flash.as_mut().read(offset, &mut buffer).await?;

        Ok(buffer)
    }
}
